import XCTest
import MixboxGenerators
import MixboxStubbing

final class CommonLogicGeneratorFacadeTests: BaseGeneratorFacadeTests {
    func test___generate___is_stateless() {
        stubDefaultConstants()
        
        var object: FinalClass
        
        object = generator.generate {
            $0.int = -3840189766160597805
            $0.string = "A4A88842-9070-4429-B7A2-AD84373B7250"
        }
        
        XCTAssertEqual(object.int, -3840189766160597805)
        XCTAssertEqual(object.string, "A4A88842-9070-4429-B7A2-AD84373B7250")
        
        // Generation in this case should be stateless
        // Generation should give an ability to fill fields partially.
        
        object = generator.generate {
            $0.string = "D83217A1-D9AD-43EA-92F4-2AF6C8CB2865"
        }
        
        XCTAssertEqual(object.int, generatedByDefault())
        XCTAssertEqual(object.string, "D83217A1-D9AD-43EA-92F4-2AF6C8CB2865")
    }
    
    // Values generated by default should be constant all the time.
    // This is done by seeding random generator with a constant.
    // This is done to make test failures reproduceable.
    func test___generate___is_reproducible() {
        let object: FinalClass = generator.generate()
        
        XCTAssertEqual(object.int, 2947667278772165694)
        XCTAssertEqual(object.string, "vUQAfGQNREuuC1ZHV")
        XCTAssertEqual(object.structure.int, 5302155820127968924)
        XCTAssertEqual(object.structure.string, "zu2")
    }
}
